/* Copyright (c) 2015 Latchable, Inc
 * 
 * Created by Vandita Sharma 10/28/2015
 *
 * latch_i2c.c
 */
 
#include "latch_i2c.h"

void i2c_gpio_init()
{
	uint32_t err_code;
	//initialize gpiote
	err_code = nrf_drv_gpiote_init();
	APP_ERROR_CHECK(err_code);

	//configure interrupt pin with a handler/event
	nrf_drv_gpiote_in_config_t in_config = GPIOTE_CONFIG_IN_SENSE_HITOLO(true);
    in_config.pull = NRF_GPIO_PIN_PULLUP;
	err_code = nrf_drv_gpiote_in_init(INTERRUPT, &in_config, i2c_event_handler);
	APP_ERROR_CHECK(err_code);
	nrf_drv_gpiote_in_event_enable(INTERRUPT, true);
}

bool i2c_transfer_data(uint8_t address, uint8_t * data, uint8_t data_length)
{
	//write or read from slave. try 4 times
	done = 0;
	tries = TRIES_I2C;
    do
    {
    	done = twi_master_transfer(address, data, data_length, TWI_ISSUE_STOP);
    }while(!done && (--tries));
    if(!done)return false;
    return true;
}

bool i2c_init(void)
{
	//write wake reason to slave. try 4 times
	done = 0;
	tries = TRIES_I2C;
    do
    {
    	done = twi_master_init();
    }while(!done && (--tries));
    if(!done)return false;
    SEGGER_RTT_WriteString(0, "init done  .\n");
    return true;
}

void i2c_transfer()
{
	command = 0x00;
	command_ret = 0x00;
    SEGGER_RTT_WriteString(0, "i2c_transfer  .\n");
    //initialize i2c.try 4 times
	//if(!i2c_init())return;
	
	//write wake reason to slave. try 4 times
    command = REQUEST_WAKE_REASON;
   	if(!i2c_transfer_data(slave_addr, &command, 1)) return;
  	nrf_delay_ms(I2C_OP_DELAY_TIME_MS);
    //read slave wake reason. try 4 times
	if(!i2c_transfer_data(slave_addr|TWI_READ_BIT, &command_ret, 1))return;

    SEGGER_RTT_WriteString(0, "disabling i2c .\n");
	NRF_TWI1->TASKS_STOP = 1;
	NRF_TWI1->ENABLE = TWI_ENABLE_ENABLE_Disabled << TWI_ENABLE_ENABLE_Pos;
	NRF_TWI1->INTENCLR = (TWI_INTENCLR_STOPPED_Disabled << TWI_INTENCLR_STOPPED_Pos)| \
	                        (TWI_INTENCLR_RXDREADY_Disabled << TWI_INTENCLR_RXDREADY_Pos)| \
	                        (TWI_INTENCLR_TXDSENT_Disabled << TWI_INTENCLR_TXDSENT_Pos)| \
	                        (TWI_INTENCLR_ERROR_Disabled << TWI_INTENCLR_ERROR_Pos)| \
	                        (TWI_INTENCLR_BB_Disabled << TWI_INTENCLR_BB_Pos);
}

//static void i2c_event_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action);
void i2c_event_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action)
{
    SEGGER_RTT_WriteString(0, "interrupt  .\n");
	int_received =1;
}


